


#
############ R LAB 21st October 2019
############ post stratification and Calibration
########### European Indicators of poverty and vulnerabilities for Sustainable
# Development Goal and Seminars


library(survey)


# Post-stratification, raking, and calibration (or GREG estimation) 
# are related ways of using auxiliary information available on the whole population. 
# These methods all involve adjusting the sampling weights so that the known population 
# totals for auxiliary variables are reproduced exactly.

##########################

# Post-stratification

#############################

# Post-stratification adjusts the sampling and replicate weights 
# so that the joint distribution of a set of post-stratifying variables
# matches the known population joint distribution

# Post-stratification allows much more flexibility in weights, in
# small samples can result in very influential points, loss of
# efficiency.

# Post-stratification is done with the postStratify() function, 
# for survey designs with or without replicate weights.


data(api)

dclus1<-svydesign(id=~dnum, weights=~pw, data=apiclus1, fpc=~fpc)
# svydesign: Specify a complex survey design.
# fpc: for the finire population corrections to varianza
# apuclus1_ is a cluster sample of school districts

rclus1<-as.svrepdesign(dclus1)
# as.svrepdesign(dclus1) convert the survey design dclus1 to use replicate weights
# Creates a replicate-weights survey design object from a traditional strata/cluster 
# survey design object. 

svymean(~api00, rclus1)
svytotal(~enroll, rclus1)

# post-stratify on school type
pop.types <- data.frame(stype=c("E","H","M"), Freq=c(4421,755,1018))
#or: pop.types <- xtabs(~stype, data=apipop)
#or: pop.types <- table(stype=apipop$stype)

rclus1p<-postStratify(rclus1, ~stype, pop.types)
summary(rclus1p)
svymean(~api00, rclus1p)
svytotal(~enroll, rclus1p)


## and for svydesign objects
dclus1p<-postStratify(dclus1, ~stype, pop.types)
summary(dclus1p)
svymean(~api00, dclus1p)
svytotal(~enroll, dclus1p)



## Calibration

# Calibration (or GREG) estimators generalise post-stratification and raking by calibrating 
# a sample to the marginal totals of variables in a linear regression model. 
# Function calibrate() reweights the survey design and adds additional information 
# that is used by svyrecvar to reduce the estimated standard errors.

#Calibration, or GREG estimation, allows continuous 
#as well as discrete auxiliary variables. 
#The method is motivated by regression estimation of a total
#but can be computed simply by reweighting, using the calibrate() function.

# The method is described for estimation of a total in 
#Srndal et al Model Assisted Survey Ssampling; 
# we use the same approach to estimate the total of estimating functions.

data(api)

? api
# Student performance in California schools

dclus1<-svydesign(id=~dnum, weights=~pw, data=apiclus1, fpc=~fpc)
# apiclus1 is a cluster sample of school districts
# dnum: disctrict number
apiclus1$pw
# The sampling weights in apiclus1 are incorrect 
# (the weight should be 757/15) but are as obtained from UCLA.
apiclus1$stype
# elementary, middle, high

pop.types <- data.frame(stype=c("E","H","M"), Freq=c(4421,755,1018))


# First we calibrate to the school type variable
# cal_names 
# The function cal_names shows what the column names of this model matrix will be
cal_names(~stype, dclus1)

pop.totals<-c(`(Intercept)`=6194, stypeH=755, stypeM=1018)

## Calibration for a single factor variable this is equivalent to
## poststratification ( postStratify )

dclus1g<-calibrate(dclus1, ~stype, pop.totals)

dclus1g

str(dclus1g)

svymean(~api00, dclus1g)
svytotal(~enroll, dclus1g)
svytotal(~stype, dclus1g)

## Make weights constant within school district
dclus1agg<-calibrate(dclus1, ~stype, pop.totals, aggregate=1)
svymean(~api00, dclus1agg)  # API in 2000
svytotal(~enroll, dclus1agg)  # number of students enrolled
svytotal(~stype, dclus1agg)  # Elementary/Middle/High School


## Now add   Met school-wide growth target?
# We can calibrate using the two variables stype and sch.wide, as we did with raking above.
cal_names(~stype+sch.wide, dclus1)
dclus1g2 <- calibrate(dclus1, ~stype+sch.wide, c(pop.totals, sch.wideYes=5122))

svymean(~api00, dclus1g2)
svytotal(~enroll, dclus1g2)
svytotal(~stype, dclus1g2)

## Finally, calibrate on 1999 API and school type
# So we can look at a calibration example with a continuous variable.

#Imagine that the 1999 API is known for all schools 
#but the 2000 API is known only for the sample. 
#Since there is a high correlation between 1999 and 2000 API 
#(see the graphics example for a scatterplot),
#calibrating to 1999 API reduces the uncertainty in mean 2000 API considerably

cor(apiclus1$api00, apiclus1$api99)

cal_names(~stype+api99, dclus1)
dclus1g3 <- calibrate(dclus1, ~stype+api99, c(pop.totals, api99=3914069))

svymean(~api00, dclus1g3)
svytotal(~enroll, dclus1g3)
svytotal(~stype, dclus1g3)


## Ratio estimators are calibration estimators
dstrat<-svydesign(id=~1,strata=~stype, weights=~pw, data=apistrat, fpc=~fpc)
svytotal(~api.stu,dstrat)

common<-svyratio(~api.stu, ~enroll, dstrat, separate=FALSE)
predict(common, total=3811472)

'pop<-3811472
## equivalent to (common) ratio estimator
dstratg1<-calibrate(dstrat,~enroll-1, pop, variance=1)
svytotal(~api.stu, dstratg1)



